function [val,grad] = obj_fn(x)
% Objective function for QP problem
% - computes function value and gradient
% - includes pref. class probabilities ('muXpH_vec') via elementwise product
%   (see footnote 20; has minimial computational burden in this case)

% Attach global variables:
global Q muXpH_vec N_alphas N_types_ego

% Elementwise product with pref. class probabilities:
am = x(1:N_alphas) .* muXpH_vec' ;

% Function value:
val = 0.5 * am' * Q * am;

% Gradient:
grad = vertcat( Q * am, zeros(2*N_types_ego,1) );

end
