function [N_alphas, N_prefcases, H_blk, H_wht, startH_full, H_list_full] = make_Q( param )
% Generates the objective matrix Q given payoff parameters.
% - find "acceptable" types for each draw of preference shocks
% - construct preference classes and their distribution
% - construct matrix S and matrix Q



%% PROGRAM SETUP

% Define global variables:

global Shocksum draws
global N_types t_index t_index_ego can_add new_types not_rel
global nX_list nX_list_drop nv_list nv_list_drop nw_list nw_list_drop


 
%% PREFERENCES FROM INDIVIDUAL DRAWS

% A) Common Values

% Preference parameters:

f.BB = param(1);
f.BW = param(2);
f.WB = param(3);
f.WW = param(4);
   v = param(5);
   w = param(6);

% Common values of types:

ubase.B = nX_list * [f.BB,f.BW]' + nv_list * v + nw_list * w;
ubase.W = nX_list * [f.WB,f.WW]' + nv_list * v + nw_list * w;

% Common values with links dropped:

udrop.B = ...
    [nX_list_drop(:,:,1) * [f.BB,f.BW]' , nX_list_drop(:,:,2) * [f.BB,f.BW]' , nX_list_drop(:,:,3) * [f.BB,f.BW]' ] ...
    + nv_list_drop * v + nw_list_drop * w;

udrop.W = ...
    [nX_list_drop(:,:,1) * [f.WB,f.WW]' , nX_list_drop(:,:,2) * [f.WB,f.WW]' , nX_list_drop(:,:,3) * [f.WB,f.WW]' ] ...
    + nv_list_drop * v + nw_list_drop * w;

% B) Evaluate Types for Each Draw of Shocks

% NOTE: Comparisons deduct small amount from alternatives to address numerical precision issues.

% Acceptable types for black ego, with each draw of preference shocks:

ok.B = ...  % Utility of each type:
	repmat(ubase.B',draws,1) + ...
    squeeze( Shocksum(:,1,1+nX_list(:,1)) ) + squeeze( Shocksum(:,2,1+nX_list(:,2)) ) ...
    >= ...  % Utility of types reached by dropping a link:
    max( max( ...
    repmat(udrop.B(:,1)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,1)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,1)) ) , ...
    repmat(udrop.B(:,2)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,2)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,2)) )  ) , ...
    repmat(udrop.B(:,3)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,3)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,3)) )  ...
    );

% Acceptable types for white ego, with each draw of preference shocks:

ok.W = ...  % Utility of each type:
	repmat(ubase.W',draws,1) + ...
    squeeze( Shocksum(:,1,1+nX_list(:,1)) ) + squeeze( Shocksum(:,2,1+nX_list(:,2)) ) ...
    >= ...  % Utility of types reached by dropping a link:
    max( max( ...
    repmat(udrop.W(:,1)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,1)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,1)) ) , ...
    repmat(udrop.W(:,2)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,2)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,2)) )  ) , ...
    repmat(udrop.W(:,3)',draws,1) - 0.00001 + ...
        squeeze( Shocksum(:,1,1+nX_list_drop(:,1,3)) ) + squeeze( Shocksum(:,2,1+nX_list_drop(:,2,3)) )  ...
    );

% Restrict to observed or adjacent types to save memory:

ok.B(:,not_rel.B) = 0;
ok.W(:,not_rel.W) = 0;



%% CONSTRUCT PREFERENCE CLASSES

% Storage objects:
% (separate lists for each ego characteristic)
% (initialize lists with pref. class containing only the isolated type)

H_blk.list = 1;
H_blk.freq = 0;
H_wht.list = 1 + N_types;
H_wht.freq = 0;

% Start loop over draws:

for s = 1:draws

% A) Pref. Classes with Black Ego

% Types in preference class for individual draw of shocks, given black ego:

H_Bi = t_index(ok.B(s,:));

% Add to black isolated class if H_Bi only contains the isolated type:

if length(H_Bi) == 1 
    H_blk(1).freq = H_blk(1).freq + 1;
end

% Otherwise compare with other black preference classes:

if length(H_Bi) > 1
    sameset = false;
    for H = 1:length(H_blk)
        if length(H_blk(H).list)==length(H_Bi)
            % Add to frequency of existing class if match is found:
            if all(H_blk(H).list==H_Bi)
                sameset = true;
                H_blk(H).freq = H_blk(H).freq + 1;
                break
            end
        end
    end
    % Add new preference class if no match is found:
    if ~sameset
        H_add.freq = 1;
        H_add.list = H_Bi;
        H_blk = [H_blk, H_add];
    end
end

% B) Pref. Classes with White Ego

% Types in preference class for individual draw of shocks, given white ego:

H_Wi = t_index(ok.W(s,:)) + N_types;

% Add to white isolated class if H_Wi only contains the isolated type:

if length(H_Wi) == 1 
    H_wht(1).freq = H_wht(1).freq + 1;
end

% Otherwise compare with other white preference classes:

if length(H_Wi) > 1
    sameset = false;
    for H = 1:length(H_wht)
        if length(H_wht(H).list)==length(H_Wi)
            % Add to frequency of existing class if match is found:
			if all(H_wht(H).list==H_Wi)
                sameset = true;
                H_wht(H).freq = H_wht(H).freq + 1;
                break
            end
        end
    end
    % Add new preference class if no match is found:
	if ~sameset
        H_add.freq = 1;
        H_add.list = H_Wi;
        H_wht = [H_wht, H_add];
    end
end

end

% C) Compile Results

% Number of preference classes:

N_prefs_blk = length(H_blk);
N_prefs_wht = length(H_wht);

N_prefcases = N_prefs_blk + N_prefs_wht;

% Sequence of types in all pref. classes:

H_list_full = [H_blk.list H_wht.list];

% Vector indicating the start of each pref. class in above sequence:

startH_full = find( H_list_full==1 | H_list_full==1+N_types );

% Number of allocation parameters:

clear global N_alphas
global N_alphas

N_alphas = length(H_list_full)



%% QUADRATIC OBJECTIVE FUNCTION

% Upper bound on number of potentially nonzero entries in S:

N_nz = sum(ismember(H_list_full,t_index_ego(can_add)));

% Storage object for list of nonzero entries in S:

S_cells = repmat(single(0),2,N_nz^2);

% A) Construct Matrix S

% Counter object:

next = 0;

% Loop over preference classes with black ego:

for H = 2:N_prefs_blk
    H_set = H_blk(H).list;
    for nt = 1:length(H_set)
        Ht = startH_full(H)-1+nt;
         t = H_set(nt);
         if can_add(t)
             % Types s that someone of type t w/pref's in H would like to add link to:
			 s_set = t_index_ego(ismember(new_types(t,:),H_set));
             if ~isempty(s_set)
				% Allocation parameters that correspond to a desired alter type:
				Hs_list = find(ismembc(H_list_full,s_set));
				% Add to list of nonzero entries in S:
				S_cells(1, next + (1:length(Hs_list)) ) = repmat(Ht,1,length(Hs_list));
				S_cells(2, next + (1:length(Hs_list)) ) = Hs_list;
				% Advance the counter:
				next = next + length(Hs_list);
             end
         end
    end
end

% Loop over preference classes with white ego:

for H = 2:N_prefs_wht
    H_set = H_wht(H).list;
    for nt = 1:length(H_set)
        Ht = startH_full(H+N_prefs_blk)-1+nt;
         t = H_set(nt);
         if can_add(t)
             % Types s that someone of type t w/pref's in H would like to add link to:
             s_set = t_index_ego(ismember(new_types(t,:),H_set));
             if ~isempty(s_set)
				% Allocation parameters that correspond to a desired alter type:
				Hs_list = find(ismembc(H_list_full,s_set));
				% Add to list of nonzero entries in S:
				S_cells(1, next + (1:length(Hs_list)) ) = repmat(Ht,1,length(Hs_list));
				S_cells(2, next + (1:length(Hs_list)) ) = Hs_list;
				% Advance the counter:
				next = next + length(Hs_list);
             end
         end
    end
end

% Define sparse matrix from list of nonzero entries:

S = sparse(double(S_cells(1,1:next)),double(S_cells(2,1:next)),true,N_alphas,N_alphas);

clear variable S_cells

% B) Construct Matrix Q

% Define variable:

clear global Q
global Q

% Hadamard product:

Q = S & S';

end
