# Use simplification of QP problem when D=1 and L=1
# - conduct (simultaneous) grid search for equilibrium vectors type shares
# - select additional equilibria from type shares found via grid search with simplified QP problem



### PROGRAM SETUP

# Load function to assess equilibrium conditions:

source("eq_typeshares.R")



### POPULATION AND PREFERENCES

# Measures of nodes with each characteristic:

mu.B <- N.B / N
mu.W <- N.W / N

ratio.BW <- mu.B/mu.W; ratio.WB <- mu.W/mu.B

# Interpret preference parameters as probabilities:
# (b/c shocks have uniform distribution; see Section 7.1)

probs <- param



### EQUILIBRIUM VECTORS OF TYPE SHARES

# A) Grid Search

# Grid to evaluate:

gridpts <- (0:50)/100

# Grid Structure:

Q.grid <- array(dim=rep(length(gridpts),4) ) 

# Storage objects (for each dimension of the parameter vector):

P.grid.BB <- P.grid.BW <- P.grid.WB <- P.grid.WW <- Q.grid

# Grid values in each dimension:

for (i in 1:length(gridpts)) P.grid.BB[i,,,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.BW[,i,,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.WB[,,i,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.WW[,,,i] <- gridpts[i]

# Feasible type shares for type (W,B):
# (must have 1:1 matches for best friendships)

S.grid.WB <- ratio.BW*P.grid.BW

# Evaluate grid of vectors of type shares to find equilibria:

Q.grid <- eq_D1L1(P.grid.BB,P.grid.BW,S.grid.WB,P.grid.WW,
                   probs[1],probs[2],probs[3],probs[4])

# Number of recovered points in search:

sum(Q.grid)



### SELECT ADDITIONAL EQUILIBRIA

# Min and max equilibrium type shares in each dimension:

s.min.BB <- min(P.grid.BB[Q.grid]); s.max.BB <- max(P.grid.BB[Q.grid])
s.min.BW <- min(P.grid.BW[Q.grid]); s.max.BW <- max(P.grid.BW[Q.grid]) 

s.min.WW <- min(P.grid.WW[Q.grid]); s.max.WW <- max(P.grid.WW[Q.grid])
s.min.WB <- min(S.grid.WB[Q.grid]); s.max.WB <- max(S.grid.WB[Q.grid]) 

# Index of unique equilibrium vectors
# (only need one slice of array of recovered vectors due to duplicates)

eq.index <- which(Q.grid[,,which(gridpts==s.min.BW),],arr.ind=T)

# Repeat shares of (B,W) to use as shares of (W,B):

eq.index <- eq.index[,c(1,2,2,3)]

# List of equilibrium type shares:

equilibria <- cbind(P.grid.BB[eq.index],P.grid.BW[eq.index],
                    S.grid.WB[eq.index],P.grid.WW[eq.index] )

# Randomly select some equilibria to use as observations:

set.seed(333)

obs.eq <- sample(1:nrow(equilibria),10)
