eq_D1L1 <- function(s.BB=s.BB,s.BW=s.BW,s.WB=s.WB,s.WW=s.WW,
                     p.BB=p.BB,p.BW=p.BW,p.WB=p.WB,p.WW=p.WW)  {
# xxx DESCRIPTION
# - evaluates a set of type shares (s.xy) and one param. vector (p.xy) or vice versa
# - here type shares are conditional on the ego characteristic ("race")
#   (i.e., they add up to 1 within each ego characteristic)

# First compute the shares of isolates:

s.B0 <- 1 - s.BB - s.BW
s.W0 <- 1 - s.WW - s.WB

# A) Bounds on Equilibrium Type Shares

# Note: See Appendix D.2.
# Bounds are found by setting allocation parameters in Table D1 to 0 or 1.

# Lower bounds on cross-race friendships:
 
s.BW.lo <- pmin(p.BW*(1-p.BB), ratio.WB*p.WB*(1-p.WW) )
s.WB.lo <- pmin(p.WB*(1-p.WW), ratio.BW*p.BW*(1-p.BB) )

# Upper bounds on cross-race friendships:
 
s.BW.hi <- pmin(p.BW, ratio.WB*p.WB)
s.WB.hi <- pmin(p.WB, ratio.BW*p.BW)

# Lower bounds on isolates:

s.B0.lo <- 1 - p.BB - pmin(p.BW*(1-p.BB), ratio.WB*p.WB)
s.W0.lo <- 1 - p.WW - pmin(p.WB*(1-p.WW), ratio.BW*p.BW)

# Upper bounds on isolates:

s.B0.hi <- 1 - p.BB - pmin(p.BW*(1-p.BB), pmax(ratio.WB*p.WB*(1-p.WW)-p.BB*p.BW,0) )
s.W0.hi <- 1 - p.WW - pmin(p.WB*(1-p.WW), pmax(ratio.BW*p.BW*(1-p.BB)-p.WW*p.WB,0) )

# B) Implied Allocation Parameters

# Infer values of two allocation parameters:
# ( 1-alpha_4(B,W) and 1-alpha_8(W,B) )

a.BB <- (s.BB - p.BB + p.BB*p.BW)/(p.BB*p.BW)
a.WW <- (s.WW - p.WW + p.WW*p.WB)/(p.WW*p.WB)

# Compute the implied shares of cross-race friendships:

s.BW.max <- p.BW - a.BB*p.BB*p.BW
s.WB.max <- p.WB - a.WW*p.WW*p.WB

s.BW.eq <- pmin(s.BW.max, ratio.WB*s.WB.max)
s.WB.eq <- pmin(s.WB.max, ratio.BW*s.BW.max)

# C) Check Conditions are Satisfied

# Note: Small adjustments are included to address numerical precision failures.

# Allocation parameters are between 0 and 1:

sat.aBB <- -0.0001 <= a.BB & a.BB <= 1.0001
sat.aWW <- -0.0001 <= a.WW & a.WW <= 1.0001

# Implied bounds are satisfied:

sat.BW <- s.BW.lo - 0.0001 <= s.BW & s.BW <= s.BW.hi + 0.0001
sat.WB <- s.WB.lo - 0.0001 <= s.WB & s.WB <= s.WB.hi + 0.0001

sat.BW <- s.BW.eq - 0.0101 <= s.BW & s.BW <= s.BW.eq + 0.0101
sat.WB <- s.WB.eq - 0.0101 <= s.WB & s.WB <= s.WB.eq + 0.0101

sat.B0 <- s.B0.lo - 0.0001 <= s.B0 & s.B0 <= s.B0.hi + 0.0001
sat.W0 <- s.W0.lo - 0.0001 <= s.W0 & s.W0 <= s.W0.hi + 0.0001

# Shares of cross-race friendships are balanced:

sat.cross <- s.WB == ratio.BW*s.BW

# Result - is it equilibrium (TRUE/FALSE):

sat.aBB & sat.aWW & sat.BW & sat.WB & sat.B0 & sat.W0 & sat.cross

}  #  END: function
