# Microsimulation procedure to find pairwise stable networks for D=1 and L=1.
# - this removes lines related to indirect connections



### PROGRAM SETUP

# Storage objects:

eq.found <- logical(sims)
init.nets <- rep(NA, sims)
cyc.counts <- rep(NA, sims)

G.sim <- array(dim=c(N,N,sims))



### MASTER LOOP

# 1) Loop over Different Networks
# (i.e., different draws of preference shocks)

for (s in 1:sims)  {

  print(c("Simulation",s) )

# Extract preference shocks:

Shocks.B <- cbind(0,Draws.sim[s,,1])
Shocks.W <- cbind(0,Draws.sim[s,,2])

###

# 2) Loop over Random Initial Graphs

# Objects to record stability and track number of initial graphs:

stable.eq <- FALSE
net.no <- 0

# Start loop:

while (stable.eq == FALSE & net.no < max.nets)  {

net.no <- net.no + 1

# Load random initial graph:

G1 <- matrix(FALSE, N, N)
G1[G.init.list[[s]][[net.no]]] <- TRUE

###

# 3) Loop to Find an Equilibrium

# Objects to count link changes and number of sequences over all pairs:

changes <- 1
counter <- 0

# Start loop:

while (changes > 0 & counter < num.seqs)  {

# Set counters:

changes <- 0
counter <- counter+1

# Draw a random sequence over all pairs:

pairnums <- sample(1:(N*(N-1)/2), replace=FALSE)

# Convert sequence of pair indices to row and column indices:

paircol <- ceiling( (sqrt(8*pairnums + 1) - 1)/2 ) + 1
pairrow <- paircol*(paircol-1)/2 - pairnums + 1

###

# 4) One Sequence through All Pairs
# (evaluate and update link status of each pair)

# Start sequence:

for (l in 1:length(pairnums) )  {

# The nodes in the pair:

  i <- pairrow[l]; j <- paircol[l]

# Links and degree of each node in the pair:

  G1i <- G1[i,]; Ni <- sum(G1i)
  G1j <- G1[j,]; Nj <- sum(G1j)

# Check bound on degree if pair is unlinked:
# (or proceed if pair is already linked)

if ( (!G1i[j] & Ni < L & Nj < L) | G1i[j] )  {

# Extract indices of nodes in subnetwork connected to this pair:
# (all nodes up to d=2 from i or j)

  N0sub <- c(i,j)  #  the nodes themselves

  Nsub <- N0sub  #  node indices for subnetwork
  Gsub <- G1[Nsub,Nsub]  #  adjacency matrix for subnetwork

# Characteristics of nodes in the subnetwork:

  Bsub <- black[Nsub]
  Wsub <- white[Nsub]

# Alternative subnetwork (change ij):

  Gsub.alt <- Gsub
  Gsub.alt[1,2] <- !Gsub.alt[1,2]; Gsub.alt[2,1] <- !Gsub.alt[2,1]

# Degree in alternative subnetwork:

  Ni.alt <- sum(Gsub.alt[1,]); Nj.alt <- sum(Gsub.alt[2,])

# Characteristics of the two nodes in the pair:

  Bi <- black[i]; Bj <- black[j]
  Wi <- white[i]; Wj <- white[j]

# Characteristics of their direct alters:

  nX <- Gsub[1:2,] %*% cbind(Bsub,Wsub)
  nX.alt <- Gsub.alt[1:2,] %*% cbind(Bsub,Wsub)

# Payoffs in current network:

U.ij <-
  # direct connections:
  c(
    f.same[i]*Ni + f.diff[i]*(Bi*nX[1,"Wsub"] + Wi*nX[1,"Bsub"]),
    f.same[j]*Nj + f.diff[j]*(Bj*nX[2,"Wsub"] + Wj*nX[2,"Bsub"]) ) +
  # preference shocks:
  diag( Shocks.B[c(i,j),nX[1:2,"Bsub"]+1] + Shocks.W[c(i,j),nX[1:2,"Wsub"]+1] )

# Payoffs in alternative network:

U.ij.alt <- 
  # direct connections:
  c(
    f.same[i]*Ni.alt + f.diff[i]*(Bi*nX.alt[1,"Wsub"] + Wi*nX.alt[1,"Bsub"]),
    f.same[j]*Nj.alt + f.diff[j]*(Bj*nX.alt[2,"Wsub"] + Wj*nX.alt[2,"Bsub"]) ) +
  # preference shocks:
  diag( Shocks.B[c(i,j),nX.alt[1:2,"Bsub"]+1] + Shocks.W[c(i,j),nX.alt[1:2,"Wsub"]+1] )

# Utility comparisons for i and j:

pref.alt.ij <- sum(U.ij.alt > U.ij)

# Myopically update the link:
 
if (!G1[i,j] & pref.alt.ij==2)  {
  G1[i,j] <- G1[j,i] <- TRUE
  changes <- changes+1
    }  else  {
if (G1[i,j] & pref.alt.ij > 0)  {
  G1[i,j] <- G1[j,i] <- FALSE
  changes <- changes+1
    }
  }

} # END if stmt for bounded degree

} # END one sequence through pairs

print(c("sequence done:", changes,"changes") )

} # END "while" loop for any link changes

stable.eq <- changes==0

} # END "while" loop over initial networks

###

# 5) Store Results

eq.found[s] <- stable.eq
init.nets[s] <- net.no
cyc.counts[s] <- counter

G.sim[,,s] <- G1

if (stable.eq)  {
  print(c("PW Stable", "Initial graphs:", net.no, "Sequences over pairs:", counter) ) 
  }  else  print("Gave Up")

} # END loop over networks (i.e., preference shocks)
