# Plot of type shares in equilibrium networks:
# - plot shares from one equilibrium network found in microsimulation
# - along with and all equilibrium vectors found in grid search (see Supplemental Appendix D.2)
# - also plots the identified sets found using the simplified QP (see Supplemental Appendix D.2)



### PLOTS OF EQUILIBRIUM TYPE SHARES

# Combined plot of type shares in selected network from microsimulation
# and all equilibrium type shares found with simplified QP problem:
# (Figure 5)

png(file="equilibria.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid] * mu.B/(mu.B+mu.W),P.grid.BW[Q.grid] * mu.B/(mu.B+mu.W), pch=19,
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BB", ylab="Shares BW" )
  points(observed.shares[2],observed.shares[3], pch=24,
  bg="grey", cex=1.25)

plot(P.grid.WW[Q.grid] * mu.W/(mu.B+mu.W), S.grid.WB[Q.grid] * mu.W/(mu.B+mu.W), pch=19,
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares WW", ylab="Shares WB" )
  points(observed.shares[6],observed.shares[5], pch=24,
  bg="grey", cex=1.25)

plot(P.grid.BW[Q.grid] * mu.B/(mu.B+mu.W), S.grid.WB[Q.grid] * mu.W/(mu.B+mu.W), pch=19,
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BW", ylab="Shares WB" )
  points(observed.shares[3],observed.shares[5], pch=24,
  bg="grey", cex=1.25)

plot(P.grid.BB[Q.grid] * mu.B/(mu.B+mu.W), P.grid.WW[Q.grid] * mu.W/(mu.B+mu.W), pch=19,
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BB", ylab="Shares WW" )
  points(observed.shares[2],observed.shares[6], pch=24,
  bg="grey", cex=1.25)

dev.off()

# Combined plot of type shares in four equilibria selected randomly
# from set of all equilibria (found with simplified QP)
# (Figure D.3)

temp <- equilibria[obs.eq[1:4],]

png(file="observed.png", height=720, width=720)

par(mfrow=c(2,2))

plot(temp[,1] * mu.B/(mu.B+mu.W), temp[,2] * mu.B/(mu.B+mu.W),
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BB", ylab="Shares BW", pch=c("A","B","C","D"), cex=1.5)

plot(temp[,4] * mu.W/(mu.B+mu.W), temp[,3] * mu.W/(mu.B+mu.W),
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares WW", ylab="Shares WB", pch=c("A","B","C","D"), cex=1.5)

plot(temp[,2] * mu.B/(mu.B+mu.W), temp[,3] * mu.W/(mu.B+mu.W),
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BW", ylab="Shares WB", pch=c("A","B","C","D"), cex=1.5)

plot(temp[,1] * mu.B/(mu.B+mu.W), temp[,4] * mu.W/(mu.B+mu.W),
  xlim=c(0, 0.3), ylim=c(0, 0.3),
  xlab="Shares BB", ylab="Shares WW", pch=c("A","B","C","D"), cex=1.5)

dev.off()



### PLOTS OF IDENTIFIED SETS FROM SIMPLIFIED QP PROBLEM

# Clean up:

rm(S.grid.WB)

# Grid to evaluate:

gridpts <- (0:50)/50

# Grid Structure:

Q.grid <- array(dim=rep(length(gridpts),4) ) 
# dimnames=list(BB=gridpts,BW=gridpts,WW=gridpts) )

# Storage objects (for each dimension of the parameter vector):

P.grid.BB <- P.grid.BW <- P.grid.WB <- P.grid.WW <- Q.grid

# Grid values in each dimension:

for (i in 1:length(gridpts)) P.grid.BB[i,,,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.BW[,i,,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.WB[,,i,] <- gridpts[i]
for (i in 1:length(gridpts)) P.grid.WW[,,,i] <- gridpts[i]

# Regions where parameters should be adjusted from grid:
# (otherwise infeasible values appear)

adjust.BW <- P.grid.BW*(1 - P.grid.BB) <= 
    ratio.WB * P.grid.WB*(1 - P.grid.WW) - P.grid.BB*P.grid.BW

adjust.WB <- P.grid.WB*(1 - P.grid.WW) <= 
    ratio.BW * P.grid.BW*(1 - P.grid.BB) - P.grid.WW*P.grid.WB

###

# A) Type Shares from Microsimulation

# Type shares conditional on ego characteristic:

sim.obs <- c(observed.shares[2:3]/sum(observed.shares[1:3]),
             observed.shares[5:6]/sum(observed.shares[4:6]) )

sim.obs[3] <- ratio.BW*sim.obs[2]
# (adjusting for precision issue)

# Shares of isolated types:

s.B0 <- 1 - sum(sim.obs[1:2])
s.W0 <- 1 - sum(sim.obs[3:4])

# Adjust parameter values as needed:

P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)

# Find identified set:

Q.grid <- eq_D1L1(sim.obs[1],sim.obs[2],sim.obs[3],sim.obs[4],
                   P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)

# Plot results:
# (Figure D.2)

png(file="../ID Set D1L1/IDset_micro.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid],P.grid.BW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=19 )
points(probs[1],probs[2], pch=23, bg="grey")

plot(P.grid.WW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=19 )
points(probs[4],probs[3], pch=23, bg="grey")

plot(P.grid.BW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=19 )
points(probs[2],probs[3], pch=23, bg="grey")

plot(P.grid.BB[Q.grid],P.grid.WW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=19 )
points(probs[1],probs[4], pch=23, bg="grey")

dev.off()

# B) Type Shares from Randomly Selected Equilibrium

# Example 1

e <- obs.eq[1]

# Shares of isolated types:

s.B0 <- 1 - sum(equilibria[e,1:2])
s.W0 <- 1 - sum(equilibria[e,3:4])

# Adjust parameter values as needed:

P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)

# Find identified set:

Q.grid <- eq_D1L1(equilibria[e,1],equilibria[e,2],equilibria[e,3],equilibria[e,4],
                   P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)

# Plot results:
# (Figures D.4 and D.5)

png(file="../ID Set D1L1/IDsetA.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid],P.grid.BW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=19 )
points(probs[1],probs[2], pch=23, bg="grey")

plot(P.grid.WW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=19 )
points(probs[4],probs[3], pch=23, bg="grey")

plot(P.grid.BW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=19 )
points(probs[2],probs[3], pch=23, bg="grey")

plot(P.grid.BB[Q.grid],P.grid.WW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=19 )
points(probs[1],probs[4], pch=23, bg="grey")

dev.off()

# Example 2

e <- obs.eq[2]

# Shares of isolated types:

s.B0 <- 1 - sum(equilibria[e,1:2])
s.W0 <- 1 - sum(equilibria[e,3:4])

# Adjust parameter values as needed:

P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)

# Find identified set:

Q.grid <- eq_D1L1(equilibria[e,1],equilibria[e,2],equilibria[e,3],equilibria[e,4],
                   P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)

# Plot results:
# (Figures D.4 and D.5)

png(file="../ID Set D1L1/IDsetB.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid],P.grid.BW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=19 )
points(probs[1],probs[2], pch=23, bg="grey")

plot(P.grid.WW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=19 )
points(probs[4],probs[3], pch=23, bg="grey")

plot(P.grid.BW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=19 )
points(probs[2],probs[3], pch=23, bg="grey")

plot(P.grid.BB[Q.grid],P.grid.WW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=19 )
points(probs[1],probs[4], pch=23, bg="grey")

dev.off()

# Example 3

e <- obs.eq[3]

# Shares of isolated types:

s.B0 <- 1 - sum(equilibria[e,1:2])
s.W0 <- 1 - sum(equilibria[e,3:4])

# Adjust parameter values as needed:

P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)

# Find identified set:

Q.grid <- eq_D1L1(equilibria[e,1],equilibria[e,2],equilibria[e,3],equilibria[e,4],
                   P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)

# Plot results:
# (Figures D.4 and D.5)

png(file="../ID Set D1L1/IDsetC.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid],P.grid.BW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=19 )
points(probs[1],probs[2], pch=23, bg="grey")

plot(P.grid.WW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=19 )
points(probs[4],probs[3], pch=23, bg="grey")

plot(P.grid.BW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=19 )
points(probs[2],probs[3], pch=23, bg="grey")

plot(P.grid.BB[Q.grid],P.grid.WW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=19 )
points(probs[1],probs[4], pch=23, bg="grey")

dev.off()

# Example 4

e <- obs.eq[4]

# Shares of isolated types:

s.B0 <- 1 - sum(equilibria[e,1:2])
s.W0 <- 1 - sum(equilibria[e,3:4])

# Adjust parameter values as needed:

P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)

# Find identified set:

Q.grid <- eq_D1L1(equilibria[e,1],equilibria[e,2],equilibria[e,3],equilibria[e,4],
                   P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)

# Plot results:
# (Figures D.4 and D.5)

png(file="../ID Set D1L1/IDsetD.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.grid],P.grid.BW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=19 )
points(probs[1],probs[2], pch=23, bg="grey")

plot(P.grid.WW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=19 )
points(probs[4],probs[3], pch=23, bg="grey")

plot(P.grid.BW[Q.grid],P.grid.WB[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=19 )
points(probs[2],probs[3], pch=23, bg="grey")

plot(P.grid.BB[Q.grid],P.grid.WW[Q.grid],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=19 )
points(probs[1],probs[4], pch=23, bg="grey")

dev.off()

###

# C) Identified Set Using Four Equilibria Jointly

# Array for intersection of identified sets:

Q.int <- array(TRUE, dim=rep(length(gridpts),4) ) 
# dimnames=list(BB=gridpts,BW=gridpts,WW=gridpts) )

# Loop over equilibria:

for (e in obs.eq[1:4])  {
  #
  s.B0 <- 1 - sum(equilibria[e,1:2])
  s.W0 <- 1 - sum(equilibria[e,3:4])
  #
  P.grid.BW[adjust.BW] <- pmax(1 - s.B0/(1 - P.grid.BB[adjust.BW]), 0)
  P.grid.WB[adjust.WB] <- pmax(1 - s.W0/(1 - P.grid.WW[adjust.WB]), 0)
  #
  Q.grid <- eq_D1L1(equilibria[e,1],equilibria[e,2],equilibria[e,3],equilibria[e,4],
                     P.grid.BB,P.grid.BW,P.grid.WB,P.grid.WW)
  print(sum(Q.grid) )
  #
  Q.int <- Q.int & Q.grid
  #
  }

# Plot results:
# (Figure 7)

png(file="../ID Set D1L1/IDset4.png", height=720, width=720)

par(mfrow=c(2,2))

plot(P.grid.BB[Q.int],P.grid.BW[Q.int],
  xlim=c(min(gridpts),max(gridpts)), ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_BW", pch=15, cex=1.5 )

plot(P.grid.WW[Q.int],P.grid.WB[Q.int],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_WW", ylab="f_WB", pch=15, cex=1.5 )

plot(P.grid.BW[Q.int],P.grid.WB[Q.int],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BW", ylab="f_WB", pch=15, cex=1.5 )

plot(P.grid.BB[Q.int],P.grid.WW[Q.int],
  xlim=c(min(gridpts),max(gridpts)),ylim=c(min(gridpts),max(gridpts)),
  xlab="f_BB", ylab="f_WW", pch=15, cex=1.5 )

dev.off()
