# Plot of type shares in stable networks from the microsimulation:
# - first combine types into categories (any same/opposite race friends, etc.)
# - produce one combined plot



### CATEGORIES OF TYPES

# A) Own-Race Friends

# Storage objects:

ownrace.B <- ownrace.W <- integer(0)

# Identify types with own-race friends:

for (t in 1:N.types.ego)  {
  same <- FALSE
  test <- A.list[,,t.list.ego[t,"A"]] * v.list.ego[t.list.ego[t,"v"],]
  #
  for (l in 1 + 1:L)  {
    if ( test[1,l] > 0 & any(test[1,l] == test[l,]) )  {
      same <- TRUE
    } }
    #
  if (same & t <= N.types)  { ownrace.B <- c(ownrace.B, t) }
  if (same & t >  N.types)  { ownrace.W <- c(ownrace.W, t) }
  }

# B) Opposite-Race Friends

# Storage objects:

opprace.B <- opprace.W <- integer(0)

# Identify types with opposite-race friends, for black egos:

for (t in 1:N.types)  {
  oppo <- FALSE
  test <- A.list[,,t.list.ego[t,"A"]] * v.list.ego[t.list.ego[t,"v"],]
  if (any(test[1 + 1:L,] == 2) )  opprace.B <- c(opprace.B, t)
  }

# Identify types with opposite-race friends, for white egos:

for (t in 1:N.types + N.types)  {
  oppo <- FALSE
  test <- A.list[,,t.list.ego[t,"A"]] * v.list.ego[t.list.ego[t,"v"],]
  if (any(test[1 + 1:L,] == 1) )  opprace.W <- c(opprace.W, t)
  }

# C) Mutual Friends

# Adj. matrices with mutual friends:

temp <- which( colSums(A.list[1+1:L,1+1:L,], dims=2) > 0 )

# Types with those adj. matrices:

mutual <- which( t.list.ego[,"A"] %in% temp )

# C) Friends of Friends

# Adj. matrices with friends of friends:

temp <- which( colSums(A.list[,(1+L+1):A.size,], dims=2) > 0 )

# Types with those adj. matrices:

hasfof <- which( t.list.ego[,"A"] %in% temp )



### PLOTS

# Combined plot for paper:
# (Figure 8)

pdf(file="D2L3_main.pdf")

par(mfrow=c(2,2) )

# Isolates:

plot(
  Type.shares.sim[eq.found,1],
  Type.shares.sim[eq.found,1+N.types],
  xlim=c(0,0.2), ylim=c(0,0.8), pch=20,
  xlab="B: isolated", ylab="W: isolated")

points(	observed.shares[1],
		observed.shares[1+N.types], pch=24, bg="grey")

# At least one own-race friend:

plot(
  rowSums(Type.shares.sim[eq.found,ownrace.B]),
  rowSums(Type.shares.sim[eq.found,ownrace.W]),
  xlim=c(0,0.2), ylim=c(0,0.8), pch=20,
  xlab="B: B friend(s)", ylab="W: W friend(s)")

points( sum(observed.shares[ownrace.B]),
        sum(observed.shares[ownrace.W]), pch=24, bg="grey")

# At least one opposite-race friend:

plot(
  rowSums(Type.shares.sim[eq.found,opprace.B]),
  rowSums(Type.shares.sim[eq.found,opprace.W]),
  xlim=c(0,0.2), ylim=c(0,0.2), pch=20,
  xlab="B: W friend(s)", ylab="W: B friend(s)")

points( sum(observed.shares[opprace.B]),
        sum(observed.shares[opprace.W]), pch=24, bg="grey")

# Mutual vs. FOFs:

plot(
  rowSums(Type.shares.sim[eq.found,hasfof]),
  rowSums(Type.shares.sim[eq.found,mutual]),
  xlim=c(0,1), ylim=c(0,1), pch=20,
  xlab="Indirect friend(s)", ylab="Mutual friend(s)")

points( sum(observed.shares[hasfof]),
        sum(observed.shares[mutual]), pch=24, bg="grey")

# title("Type Shares in Network Simulation", outer = TRUE)

dev.off()
