# Find the network type of each individual in each simulated network.
# - then compute type shares
# - randomly select one stable network and export shares to MATLAB



### FIND TYPES IN EACH NETWORK

# A) Preliminary Tool

# N/A

# B) Loop over Networks

# Storage object:
# (default type is isolated)

Types.micro.sim <- matrix(1, nrow=sims,ncol=N)
Types.micro.sim[,white] <- 1 + N.types  # (isolated type with white ego)

# Start of loop over networks:

for (s in 1:sims)  {

print(paste("Sim",s) )

# Global adjacency matrix:

  G1 <- G.sim[,,s] 

# Adj. matrix with extra blank row/col:
# (used to add rows for absent alters in network types)

  G1.plus <- rbind(cbind(G1,F),F)

# C) Extract Local Subnetwork for Each Node

# Loop over non-isolated nodes

for (i in which(rowSums(G1) > 0) ) {

# C.1) Direct Alters

# Indices of direct alters:

  N1sub <- c(i,which(G1[i,]))

# Add dummy index (N+1) for absent direct alters:

  Ni <- sum(G1[i,])

  N1sub <- c(N1sub, rep(N+1, L-Ni) )

# C.2) Indirect Alters

# N/A

# Nodes in local subnetwork:

Nsub <- N1sub

# Adj. matrix of local subnetwork:

A.i <- G1.plus[Nsub,Nsub]

# Characteristics of nodes in local subnetwork:

Bsub <- black[Nsub]
Wsub <- white[Nsub]

# Define vector of characteristics:

v.i <- rep(0,A.size)

v.i[Bsub] <- 1
v.i[Wsub] <- 2

# Use elementwise product to indicate node characteristics:

Av.i <- A.i * v.i

# D) Compare with List of Types

# Initialize type index:

t.index.i <- 0

# Indicator for isomorphism found:

same <- F

# Start loop over list of types:
  for (r in 1:N.types.ego)  {

# Check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  if (all(Av.r==Av.i) )  same <- T

# Record type index of node i:

if (same) t.index.i <- r

}  #  END: first if statement for 'same' not already found

}  #  END: loop over list of types

# Store and print results:

Types.micro.sim[s,i] <- t.index.i

print( paste("Network",s, "Player",i, "is type",t.index.i) )

} # END for loop over players

} # END for loop over networks



### COMPUTE TYPE SHARES

# Storage object:

Type.shares.sim <- matrix(0, nrow=sims, ncol=N.types.ego)

# Compute shares in each network:

for (s in 1:sims)  {  for (t in 1:N.types.ego)  {
  Type.shares.sim[s,t] <- sum(Types.micro.sim[s,] == t) / N
  } }



### SELECT NETWORK TO ANALYZE

# A) Type Shares in Selected Network

# Note: For this exercise we used the network that was closest to the average type shares.

# Compute Euclidean distance from average:

temp <- sqrt( colSums( 
  ( t(Type.shares.sim) - apply(Type.shares.sim[eq.found,], 2, mean) )^2
  ) )

s <- which.min(temp)

# Type shares in selected network:

observed.shares <- Type.shares.sim[s,]

# Compare type shares in selected network with mean and max:

cbind(t.list.ego,
  round(apply(Type.shares.sim[eq.found,], 2, mean)*N,2),
  apply(Type.shares.sim[eq.found,], 2, max)*N, observed.shares*N,
  apply(Type.shares.sim[eq.found,], 2, max) == observed.shares   )

# B) List of Irrelevant Types

# N/A

# C) Export to Matlab:

writeMat("../ID Set D1L1/observed.mat",
  obs_shares=observed.shares)
