# Lists of relevant features of network types and results of dropping or adding links:
# - lists containing the payoff-relevant attributes of each type
# - lists containing the payoff-relevant attributes of types reached by dropping links
# - matrix that gives the resulting type when a link is added between two types
# - export material to a MATLAB file
# - all lines related to multiple alters and indirect alters are removed



### EXTRACT PAYOFF RELEVANT ATTRIBUTES

# Note: These objects are for list of types w/out ego characteristics.

# A) Characteristics of Direct Connections

# Indicators for characteristics of direct alters:

v.list.B <- v.list[,1 + 1:L] == 1
v.list.W <- v.list[,1 + 1:L] == 2

# Number of direct alters with each characteristic:

nX.list <- matrix(0, N.types, chars)
  colnames(nX.list) <- c("B","W")

nX.list[,"B"] <- t(A.list[1,1+1:L,t.list[,"A"]]) * v.list.B[t.list[,"v"] ]

nX.list[,"W"] <- t(A.list[1,1+1:L,t.list[,"A"]]) * v.list.W[t.list[,"v"] ]

# B) Number of Indirect Connections

# N/A

# C) Number of Mutual Connections

# N/A



### ATTRIBUTES OF TYPES REACHED BY DROPPING LINKS

# A) Storage Objects

# Adjacency matrices and paths:

 A.list.drop <- A.list

# Alter characteristics:

nX.list.drop <- nX.list

# Start loop over links to drop:

for (l in 1:L)  {  

# A) Direct Connections

# Drop the link:

for (A in 1:N.A)  A.list.drop[1,1+l,A] <- A.list.drop[1+l,1,A] <- F

# Number of direct alters with each characteristic:

nX.list.drop[,"B" ] <- t(A.list.drop[1,1+1:L,t.list[,"A"] ]) * v.list.B[t.list[,"v"] ]

nX.list.drop[,"W" ] <- t(A.list.drop[1,1+1:L,t.list[,"A"] ]) * v.list.W[t.list[,"v"] ]

# B) Indirect Connections

# N/A

# C) Mutual Connections

# N/A

}  #  END: Loop over direct links to drop.



### ADDING LINKS

# A) Preliminary Tools

# Indicator for types that can add a link:

can.add <- colSums(A.list[1,,t.list.ego[,"A"]]) < L

# Permutation to move ego to first alter position:
# (and eliminate all other entries by using the third alter position, which is blank.

P.0e <-  diag(A.size)[c(2,1),]

# B) Loop over Type Pairs to Find Resulting Type

# Storage object:
# (the matrix described in Appendix D.5.3)

new.types <- matrix(nrow=N.types.ego, ncol=N.types.ego)

# Loop:

for (t in which(can.add) )  {
for (s in which(can.add) )  {

# Initial types (A,v representations):

  A.t <- A.list[,,t.list.ego[t,"A"]]; v.t <- v.list.ego[t.list.ego[t,"v"],]
  A.s <- A.list[,,t.list.ego[s,"A"]]; v.s <- v.list.ego[t.list.ego[s,"v"],]

# B.1) New Type of Former Type t
# (A,v representation, but not necessarily the canonical one):

# Insert former type s as first alter:

A.tbar <- A.t + P.0e %*% A.s %*% t(P.0e)

# Add link to that new alter:

A.tbar[1,2] <- A.tbar[2,1] <- TRUE

# New vector of characteristics:

v.tbar <- v.t
v.tbar[2] <- v.s[1]

# Use elementwise product to indicate node characteristics:

Av.tbar <- A.tbar * v.tbar

# B.2) Find Matching (A,v) in List of Canonical Representations

# Indicator for isomorphism found:

same <- F

# Start loop over list of types:
  for (r in 1:N.types.ego)  {

# Check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  if (all(Av.r==Av.tbar) )  same <- T

# Record index of resulting type:

if (same) new.types[t,s] <- r

}  #  END: first if statement for 'same' not already found

}  #  END: loop over list of types

print(paste(t, s, new.types[t,s]) )

} }  #  END: loop over types t and s

# Check that all resulting types have been found:

sum(is.na(new.types[can.add,can.add]))



### EXPORT TO MATLAB

library(R.matlab)

# Replace NA with 0:

new.types[is.na(new.types)] <- 0

# writeMat("envir.mat",
  # t_index=t.index, N_types=N.types,
  # )

writeMat("../ID Set D1L1/export.mat", 
  draws=draws, Shocksum=Shocksum,
  N=N, N_B=N.B, N_W=N.W,
  N_types=N.types, N_types_ego=N.types.ego, t_index=t.index, t_index_ego=t.index.ego,
  nX_list=nX.list,
  nX_list_drop=nX.list.drop,
  can_add=can.add*1, new_types=new.types
  )
