# Find the network type of each individual in each simulated network.
# - then compute type shares
# - randomly select one stable network and export shares to MATLAB



### FIND TYPES IN EACH NETWORK

# A) Preliminary Tool

# Another useful permutation:

P.231 <- diag(A.size)[c(1,3,4,2,7,8,9,10,5,6),]

# B) Loop over Networks

# Storage object:
# (default type is isolated)

Types.micro.sim <- matrix(1, nrow=sims,ncol=N)
Types.micro.sim[,white] <- 1 + N.types  # (isolated type with white ego)

# Start of loop over networks:

for (s in 1:sims)  {

print(paste("Sim",s) )

# Global adjacency matrix:

  G1 <- G.sim[,,s] 

# Adj. matrix with extra blank row/col:
# (used to add rows for absent alters in network types)

  G1.plus <- rbind(cbind(G1,F),F)

# C) Extract Local Subnetwork for Each Node

# Loop over non-isolated nodes

for (i in which(rowSums(G1) > 0) ) {

# C.1) Direct Alters

# Indices of direct alters:

  N1sub <- c(i,which(G1[i,]))

# Add dummy index (N+1) for absent direct alters:

  Ni <- sum(G1[i,])

  N1sub <- c(N1sub, rep(N+1, L-Ni) )

# C.2) Indirect Alters

# Storage object:

N2sub <- integer(0)

# Loop over direct alters:

for (l in 1:L)  {
# Find their neighbors:
  Njsub <- which(G1.plus[N1sub[1+l],])
# Add non-redundant neighbors:
  N2sub <- c(N2sub, Njsub[!(Njsub %in% c(N1sub,N2sub))] )
# Add dummy index (N+1) for absent indirect alters:
  N2sub <- c(N2sub, rep(N+1, l*(L-1) - length(N2sub) ) )
  }

# Nodes in local subnetwork:

Nsub <- c(N1sub, N2sub)

# Adj. matrix of local subnetwork:

A.i <- G1.plus[Nsub,Nsub]

# Characteristics of nodes in local subnetwork:

Bsub <- black[Nsub]
Wsub <- white[Nsub]

# Define vector of characteristics:

v.i <- rep(0,A.size)

v.i[Bsub] <- 1
v.i[Wsub] <- 2

# Placeholder value for characteristics of indirect alters:

v.i[(1+L+1):A.size] <- chars + 1

# Drop multiple paths to indirect alters (which are irrelevant):
# Note: Given the canonical representation scheme, these appear in a certain region of A.

for (l in 1:L)  {
  others <- ((1 + L + 1):A.size)[-((l-1)*2 + 1:2)]
  A.i[1+l,others] <- F; A.i[others,1+l] <- F
  }

# Drop connections among indirect alters (which are irrelevant):

A.i[(1 + L + 1):A.size,(1 + L + 1):A.size] <- F

# Use elementwise product to indicate node characteristics:

Av.i <- A.i * v.i

# D) Compare with List of Types

# Initialize type index:

t.index.i <- 0

# Indicator for isomorphism found:

same <- F

# Start loop over list of types:
  for (r in 1:N.types.ego)  {

# Check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  if (all(Av.r==Av.i) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.312 %*% Av.r %*% t(P.312)
  if (all(Av.r==Av.i) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.231 %*% Av.r %*% t(P.231)
  if (all(Av.r==Av.i) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.213 %*% Av.r %*% t(P.213)
  if (all(Av.r==Av.i) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.132 %*% Av.r %*% t(P.132)
  if (all(Av.r==Av.i) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.321 %*% Av.r %*% t(P.321)
  if (all(Av.r==Av.i) )  same <- T

} } } } } # END: inner if statements for 'same' not already found

# Record type index of node i:

if (same) t.index.i <- r

}  #  END: first if statement for 'same' not already found

}  #  END: loop over list of types

# Store and print results:

Types.micro.sim[s,i] <- t.index.i

print( paste("Network",s, "Player",i, "is type",t.index.i) )

} # END for loop over players

} # END for loop over networks



### COMPUTE TYPE SHARES

# Storage object:

Type.shares.sim <- matrix(0, nrow=sims, ncol=N.types.ego)

# Compute shares in each network:

for (s in 1:sims)  {  for (t in 1:N.types.ego)  {
  Type.shares.sim[s,t] <- sum(Types.micro.sim[s,] == t) / N
  } }



### SELECT NETWORK TO ANALYZE

# A) Type Shares in Selected Network

# Select one network at random:

s <- sample(which(eq.found), 1)

# Type shares in selected network:

observed.shares <- Type.shares.sim[s,]

# Compare type shares in selected network with mean and max:

cbind(t.list.ego,
  round(apply(Type.shares.sim[eq.found,], 2, mean)*N,2),
  apply(Type.shares.sim[eq.found,], 2, max)*N, observed.shares*N,
  apply(Type.shares.sim[eq.found,], 2, max) == observed.shares   )

# B) List of Irrelevant Types

# Note: These types are not observed in the selected network, and 
# cannot be reached by adding a link between any two observed types.
# See Section 7.2 in the article for a brief discussion.
  
# Unobserved types that could be reached by adding a link:
could.become.add <- unique(sort(new.types[observed.shares > 0, observed.shares > 0]))

# Unobserved types that could be reached by dropping a link:

could.become.drop <- unique(sort(arrayInd(
  which( new.types %in% which(observed.shares > 0) ), dim(new.types) )[,1] 
  ) )

# Note: Recall that the matrix new.types tells what type the row becomes if linked to the column.

# All unobserved types that are also relevant:

also.relevant <- unique(c(
   could.become.add[ could.become.add %in% which(observed.shares == 0) ],
  could.become.drop[could.become.drop %in% which(observed.shares == 0) ]
  ) )

# Types that are not relevant:
# (i.e., not observed or cannot be reached by adding or dropping a link)

not.relevant <- !(observed.shares > 0 | 1:N.types.ego %in% also.relevant)

# C) Export to Matlab:

writeMat("../ID Set D2L3/observed.mat",
  obs_shares=observed.shares, not_relevant=not.relevant*1)
