# Lists of relevant features of network types and results of dropping or adding links:
# - lists containing the payoff-relevant attributes of each type
# - lists containing the payoff-relevant attributes of types reached by dropping links
# - matrix that gives the resulting type when a link is added between two types
# - export material to a MATLAB file



### EXTRACT PAYOFF RELEVANT ATTRIBUTES

# Note: These objects are for list of types w/out ego characteristics.

# A) Characteristics of Direct Connections

# Indicators for characteristics of direct alters:

v.list.B <- v.list[,1 + 1:L] == 1
v.list.W <- v.list[,1 + 1:L] == 2

# Number of direct alters with each characteristic:

nX.list <- matrix(0, N.types, chars)
  colnames(nX.list) <- c("B","W")

nX.list[,"B"] <- rowSums(
  t(A.list[1,1+1:L,t.list[,"A"]]) * v.list.B[t.list[,"v"],] )

nX.list[,"W"] <- rowSums(
  t(A.list[1,1+1:L,t.list[,"A"]]) * v.list.W[t.list[,"v"],] )

# B) Number of Indirect Connections

# Storage object:

A2.list <- A.list

# Number of paths of length 2 in each adj. matrix:

for (A in 1:N.A)  A2.list[,,A] <- A.list[,,A] %*% A.list[,,A] 
  
# Count number of indirect connections:
# (indirect alters with at least one path to the ego)

nv.list <- rowSums( t(A2.list[1,(1+L+1):A.size,t.list[,"A"]]) > 0 )

# C) Number of Mutual Connections

# Storage object:

A3.list <- A.list

# Number of paths of length 3 in each adj. matrix:

for (A in 1:N.A)  A3.list[,,A] <- A.list[,,A] %*% A.list[,,A] %*% A.list[,,A] 

# Count number of mutual connections:
# (each mutual connection yields two cycles of length 3 that contain the ego)

nw.list <- A3.list[1,1,t.list[,"A"]] / 2



### ATTRIBUTES OF TYPES REACHED BY DROPPING LINKS

# A) Storage Objects

# Adjacency matrices and paths:

 A.list.drop <- A.list
A2.list.drop <- A2.list
A3.list.drop <- A3.list

# Alter characteristics, indirect connections, and mutual connections:

nX.list.drop <- nX.list
nv.list.drop <- nv.list
nw.list.drop <- nw.list

# Loop to add dimensions for each link to drop:

for (l in 2:L)  {
   A.list.drop <- abind( A.list.drop,  A.list, along=4)
  A2.list.drop <- abind(A2.list.drop, A2.list, along=4)  
  A3.list.drop <- abind(A3.list.drop, A3.list, along=4)  
  #
  nX.list.drop <- abind(nX.list.drop, nX.list, along=3)
  nv.list.drop <- abind(nv.list.drop, nv.list, along=2)
  nw.list.drop <- abind(nw.list.drop, nw.list, along=2)
  }

###

# Start loop over links to drop:

for (l in 1:L)  {  

# A) Direct Connections

# Drop the link:

for (A in 1:N.A)  A.list.drop[1,1+l,A,l] <- A.list.drop[1+l,1,A,l] <- F

# Number of direct alters with each characteristic:

nX.list.drop[,"B",l] <- rowSums(
  t(A.list.drop[1,1+1:L,t.list[,"A"],l]) * v.list.B[t.list[,"v"],] )

nX.list.drop[,"W",l] <- rowSums(
  t(A.list.drop[1,1+1:L,t.list[,"A"],l]) * v.list.W[t.list[,"v"],] )

# B) Indirect Connections

# Number of paths of length 2 in each adj. matrix:

for (A in 1:N.A)  A2.list.drop[,,A,l] <- A.list.drop[,,A,l] %*% A.list.drop[,,A,l] 
  
# Count number of indirect connections:

nv.list.drop[,l] <- rowSums( t(A2.list.drop[1,(1+L+1):A.size,t.list[,"A"],l]) > 0 )

# C) Mutual Connections

# Number of paths of length 3 in each adj. matrix:

for (A in 1:N.A)  A3.list.drop[,,A,l] <- A.list.drop[,,A,l] %*% A.list.drop[,,A,l] %*% A.list.drop[,,A,l] 

# Count number of mutual connections:

nw.list.drop[,l] <- A3.list.drop[1,1,t.list[,"A"],l] / 2

}  #  END: Loop over direct links to drop.



### ADDING LINKS

# A) Preliminary Tools

# Indicator for types that can add a link:

can.add <- colSums(A.list[1,,t.list.ego[,"A"]]) < L

# Permutation to move ego to first alter position:
# (and eliminate all other entries by using the third alter position, which is blank.

P.0e <-  diag(A.size)[c(4,1,4,4,2,3,9,10,9,10),]

# Multiple permutation to open the first alter slot:
# (to add the new alter)

P.312 <- diag(A.size)[c(1,4,2,3,9,10,5,6,7,8),]

# B) Loop over Type Pairs to Find Resulting Type

# Storage object:
# (the matrix described in Appendix D.5.3)

new.types <- matrix(nrow=N.types.ego, ncol=N.types.ego)

# Loop:

for (t in which(can.add) )  {
for (s in which(can.add) )  {

# Initial types (A,v representations):

  A.t <- A.list[,,t.list.ego[t,"A"]]; v.t <- v.list.ego[t.list.ego[t,"v"],]
  A.s <- A.list[,,t.list.ego[s,"A"]]; v.s <- v.list.ego[t.list.ego[s,"v"],]

# B.1) New Type of Former Type t
# (A,v representation, but not necessarily the canonical one):

# Insert former type s as first alter:

A.tbar <- P.312 %*% A.t %*% t(P.312) + P.0e %*% A.s %*% t(P.0e)

# Add link to that new alter:

A.tbar[1,2] <- A.tbar[2,1] <- TRUE

# Remove irrelevant links among former type s's alters:

A.tbar[(1+L+1):A.size,(1+L+1):A.size] <- FALSE

# New vector of characteristics:

v.tbar <- as.vector(P.312 %*% v.t)
v.tbar[2] <- v.s[1]

# Use elementwise product to indicate node characteristics:

Av.tbar <- A.tbar * v.tbar

# B.2) Find Matching (A,v) in List of Canonical Representations

# Indicator for isomorphism found:

same <- F

# Start loop over list of types:
  for (r in 1:N.types.ego)  {

# Check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  if (all(Av.r==Av.tbar) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.312 %*% Av.r %*% t(P.312)
  if (all(Av.r==Av.tbar) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.213 %*% Av.r %*% t(P.213)
  if (all(Av.r==Av.tbar) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.132 %*% Av.r %*% t(P.132)
  if (all(Av.r==Av.tbar) )  same <- T

# Reorder direct alters and check for match:

if (!same)  {
  Av.r <- A.list[,,t.list.ego[r,"A"]] * v.list.ego[t.list.ego[r,"v"],]
  Av.r <- P.321 %*% Av.r %*% t(P.321)
  if (all(Av.r==Av.tbar) )  same <- T

} } } } # END: inner if statements for 'same' not already found

# Record index of resulting type:

if (same) new.types[t,s] <- r

}  #  END: first if statement for 'same' not already found

}  #  END: loop over list of types

print(paste(t, s, new.types[t,s]) )

} }  #  END: loop over types t and s

# Check that all resulting types have been found:

sum(is.na(new.types[can.add,can.add]))



### EXPORT TO MATLAB

library(R.matlab)

# Replace NA with 0:

new.types[is.na(new.types)] <- 0

# writeMat("envir.mat",
  # t_index=t.index, N_types=N.types,
  # )

writeMat("../ID Set D2L3/export.mat", 
  draws=draws, Shocksum=Shocksum,
  N=N, N_B=N.B, N_W=N.W,
  N_types=N.types, N_types_ego=N.types.ego, t_index=t.index, t_index_ego=t.index.ego,
  nX_list=nX.list, nv_list=nv.list, nw_list=nw.list,
  nX_list_drop=nX.list.drop, nv_list_drop=nv.list.drop, nw_list_drop=nw.list.drop,
  can_add=can.add*1, new_types=new.types
  )

# xxx CHANGED EXPORT
