# Enumerate types (i.e., construct list of (A,v) representations)
# - construct non-isomorphic adjacency matrices
# - generate vectors of possible node characteristics
# - combine the above and check for isomorphisms to compile the list
# - draw preference shocks used later to simulate preference class probabilities



### PROGRAM SETUP

library(abind)



### ADJACENCY MATRICES

# Size of adjacency matrix:

A.size <- 1
  for ( d in 1:D ) A.size <- A.size + L*(L-1)^(d-1)

# Start list of non-isomorphic adj. matrices with the isolated type:

A.list <- matrix(F,nrow=A.size,ncol=A.size)

# A) Loop to Construct Matrices

# Loop over number of links of the ego (l.e):
  for (l.e in 1:L)  {

# A.1) No Mutual Friends

# Loop over number of additional alters of the first potential direct alter (l.1):
  for (l.1 in 0:(if (l.e >= 1) L-1 else 0) )  {
# Loop over number of additional alters of the second potential direct alter (l.2):
  for (l.2 in 0:(if (l.e >= 2) l.1 else 0) )  {
# Loop over number of additional alters of the third potential direct alter (l.3):
  for (l.3 in 0:(if (l.e >= 3) l.2 else 0) )  {
# (Note: Alters are ordered by their number of additional alters.)

# First define empty adjacency matrix:

A.t <- matrix(F,nrow=A.size,ncol=A.size)

# Add ego's links:

A.t[1, 1 + 1:l.e] <- T

# Add direct alters' other links:

if (l.1 > 0) A.t[1+1, 1 + L + 0*(L-1) + 1:l.1] <- T
if (l.2 > 0) A.t[1+2, 1 + L + 1*(L-1) + 1:l.2] <- T
if (l.3 > 0) A.t[1+3, 1 + L + 2*(L-1) + 1:l.3] <- T

# Append new A to list of adjacency matrices:

A.list <- abind(A.list,A.t,along=3)
 
} } }  #  END: loops over direct alters

# A.2) One Mutual Friendship
# (link between first and second direct alters)

# If statement for 2+ direct alters:

if (l.e >= 2)  {

# Loop over number of additional alters of the third potential direct alter (l.3):
  for (l.3 in 0:(if (l.e >= 3) L-1 else 0) )  {
# (Note: Third direct alter does not have a mutual link so it can have up to L-1 additional alters.)

# Loop over number of additional alters of the first potential direct alter (l.1):
  for (l.1 in 0:(if (l.e >= 1) L-2 else 0) )  {
# Loop over number of additional alters of the second potential direct alter (l.2):
  for (l.2 in 0:(if (l.e >= 2) l.1 else 0) )  {
# (Note: First and second alters are linked, hence they have up to L-2 additional alters.)

# First define empty adjacency matrix:

A.t <- matrix(F,nrow=A.size,ncol=A.size)

# Add ego's links:

A.t[1, 1 + 1:l.e] <- T

# Add mutual friendship link:

A.t[1+1, 1+2] <- T

# Add direct alters' other links:

if (l.1 > 0) A.t[1+1, 1 + L + 0*(L-1) + 1:l.1] <- T
if (l.2 > 0) A.t[1+2, 1 + L + 1*(L-1) + 1:l.2] <- T
if (l.3 > 0) A.t[1+3, 1 + L + 2*(L-1) + 1:l.3] <- T

# Append new A to list of adjacency matrices:

A.list <- abind(A.list,A.t,along=3)

} } }  #  END: loops over direct alters
}  #  END: if 2+ direct alters

# A.3) Two Mutual Friendships
# (first and second direct alters are both linked to the third)

# If statement for 3+ direct alters:

if (l.e >= 3)  {

# Third direct alter has no additional alters
  l.3 <- L - 3

# Loop over number of additional alters of the first potential direct alter (l.1):
  for (l.1 in 0:(if (l.e >= 1) L-2 else 0) )  {
# Loop over number of additional alters of the second potential direct alter (l.2):
  for (l.2 in 0:(if (l.e >= 2) l.1 else 0) )  {
# (Note: First and second alters are linked to the third, hence they have up to L-2 additional alters.)

# First define empty adjacency matrix:

A.t <- matrix(F,nrow=A.size,ncol=A.size)

# Add ego's links:

A.t[1, 1 + 1:l.e] <- T

# Add mutual friendship links:

A.t[1+1, 1+3] <- T
A.t[1+2, 1+3] <- T

# Add direct alters' other links:

if (l.1 > 0) A.t[1+1, 1 + L + 0*(L-1) + 1:l.1] <- T
if (l.2 > 0) A.t[1+2, 1 + L + 1*(L-1) + 1:l.2] <- T

# Append new A to list of adjacency matrices:

A.list <- abind(A.list,A.t,along=3)

} }  #  END: loops over direct alters
}  #  END: if 3+ direct alters

# A.3) Three Mutual Friendships
# (all three direct alters are mutually linked)

# If statement for 3+ direct alters:

if (l.e >= 3)  {

# First define empty adjacency matrix:

A.t <- matrix(F,nrow=A.size,ncol=A.size)

# Add ego's links:

A.t[1, 1 + 1:l.e] <- T

# Add mutual friendship links:

A.t[1+1, 1+2] <- T
A.t[1+1, 1+3] <- T
A.t[1+2, 1+3] <- T

# Append new A to list of adjacency matrices:

A.list <- abind(A.list,A.t,along=3)

}  #  END: if 3+ direct alters

# Close main loop:

} # END: loop over l.e

# Number of distinct adjacency matrices:

N.A <- dim(A.list)[3]

###

# B) Make Matrices Symmetric

for (A in 1:N.A)  {
  A.t <- A.list[,,A]
  A.list[,,A] <- A.t | t(A.t)
  }



### NODE CHARACTERISTICS

# A) Characteristics of Direct Alters

# Define storage object:

v.list <- numeric(0)

# Loop over characteristics:

# Ego is left blank:
  v1 <- 0
# First direct alter:
  for (v2 in 1:chars)  {
# Second direct alter:
  for (v3 in 1:chars)  {
# Third direct alter:
  for (v4 in 1:chars)  {
# Placeholder for indirect alters:
  v.ind <- rep(chars+1, A.size - L - 1)

# Append to list of vectors of characteristics:

v.list <- rbind(v.list, c(v1,v2,v3,v4,v.ind) )

} } }  #  END: loops over characteristics of direct alters

# Number of different vectors of characteristics of direct alters:

N.v <- nrow(v.list)

# B) Add Characteristic(s) of Ego

# Define storage object:

v.list.ego <- numeric(0)

# Loop over characteristics of ego:

for (x in 1:chars)  {
  v.list.add <- v.list
  v.list.add[,1] <- x
  v.list.ego <- rbind(v.list.ego, v.list.add)
}



### ENUMERATE TYPES W/OUT EGO CHARACTERISTICS

# A) Permutations of Direct Alters
# (to assess isomorphisms)

# Binary exchanges only needed for now:
# (numbers indicate revised order of direct alters 1,2,3)

P.132 <- diag(A.size)[c(1,2,4,3,5,6,9,10,7,8),]
P.321 <- diag(A.size)[c(1,4,3,2,9,10,7,8,5,6),]
P.213 <- diag(A.size)[c(1,3,2,4,7,8,5,6,9,10),]

# B) Loop over A's and v's to Make Enumeration

# Note: This makes a list of the indices for A and v from A.list and v.list.

# Start with isolated type to define storage object:

t.list <- rbind(c(1,1) )

colnames(t.list) <- c("A","v")

# Start loop:

for (A in 2:N.A)  {

for (v in 1:N.v)  {

# Use elementwise product to indicate node characteristics:

Av <- A.list[,,A] * v.list[v,]

# C) Check for Isomorphisms

# Indicator for isomorphism found:

same <- F

# Loop over previous vectors of alter characteristics:
  if (v > 1)  {  for (w in 1:(v-1) )  {  

# Check for match:

if (!same)  {
  Aw <- A.list[,,A] * v.list[w,]
  if (all(Aw==Av) )  same <- T

# Exchange alters 1 and 2 and check for match:

if (!same)  {
  Aw <- A.list[,,A] * v.list[w,]
  Aw <- P.213 %*% Aw %*% t(P.213)
  if (all(Aw==Av) )  same <- T

# Exchange alters 1 and 3 and check for match:

if (!same)  {
  Aw <- A.list[,,A] * v.list[w,]
  Aw <- P.321 %*% Aw %*% t(P.321)
  if (all(Aw==Av) )  same <- T

# Exchange alters 2 and 3 and check for match:

if (!same)  {
  Aw <- A.list[,,A] * v.list[w,]
  Aw <- P.132 %*% Aw %*% t(P.132)
  if (all(Aw==Av) )  same <- T

} } } }  # END: if statements for 'same' already found

} }  #  END: loop over previous vectors of alter characteristics

  if (!same) t.list <- rbind(t.list, c(A,v) )

} }

# Type indices (without ego characteristics):

N.types <- nrow(t.list)
t.index <- 1:N.types



### ENUMERATE TYPES WITH EGO CHARACTERISTICS

# Previous list provides list for first value of ego characteristic:
# (using v.list.ego rather than v.list)

t.list.ego <- t.list

# Loop over remaining possible characteristics of ego:

for (x in 1:(chars-1))  {
  t.list.add <- t.list
  t.list.add[,"v"] <- t.list[,"v"] + N.v*x
  t.list.ego <- rbind(t.list.ego, t.list.add)
}

# Type indices (with ego characteristics):

N.types.ego <- nrow(t.list.ego)
t.index.ego <- 1:N.types.ego



### PREFERENCE SHOCKS

# Number of draws and random seed:

draws <- 10000

set.seed(746257)

# Draw shocks:

Shocks <- array(rnorm(draws*L*chars), dim=c(draws,chars,L))
  dimnames(Shocks)[[2]] <- c("B","W")

# Sort shocks within each alter characteristic:
# Note: See Appendix D.3 for discussion.

for (s in 1:draws)  {
  Shocks[s,"B",1:L] <- sort(Shocks[s,"B",1:L], decreasing=TRUE)
  Shocks[s,"W",1:L] <- sort(Shocks[s,"W",1:L], decreasing=TRUE)
  }

# Take sum of shocks within each characteristic:
# (only the sum is needed to compute utility)

Shocksum <- Shocks

for (l in 1:L)  {
  Shocksum[,"B",l] <- rowSums(cbind(Shocks[,"B",1:l]) ) 
  Shocksum[,"W",l] <- rowSums(cbind(Shocks[,"W",1:l]) ) 
  }

# Add a column for zero shocks (no direct alters with those characteristics):

Shocksum <- abind(matrix(0,draws,chars),Shocksum,along=3)

# Clean up:

rm(Shocks)
